
local function setFontString(parent, fontName, fontHeight, fontStyle)
  if not parent then return nil end
	local fs = parent:CreateFontString(nil, 'OVERLAY')
	fs:SetFont(fontName, fontHeight, fontStyle)
	fs:SetJustifyH('LEFT')
	fs:SetShadowColor(0,0,0)
	fs:SetShadowOffset(1, -1)
	return fs
end

local function CalcArc(unit,cur,max)
  if (unit == "vehicle" ) then unit = "player" end
  
  local p = cur/max
  local tex_gap_top    = oUF_Drathal_HUD_Layout.Gaps[unit].t
  local tex_gap_bottom = oUF_Drathal_HUD_Layout.Gaps[unit].b
  
  local tex_gap_top_p    = tex_gap_top / oUF_Drathal_HUD_Layout.arcHeight   
  local tex_gap_bottom_p = tex_gap_bottom / oUF_Drathal_HUD_Layout.arcHeight
  local h = (oUF_Drathal_HUD_Layout.arcHeight - tex_gap_top - tex_gap_bottom) * p
  
  local top    = 1-(p-(tex_gap_top_p));
  local bottom = 1-tex_gap_bottom_p;
  top = top  - ((tex_gap_top_p+tex_gap_bottom_p)*(1-p));  
 
  return {  
    ["gapBottom"]        = tex_gap_bottom,
    ["height"]           = h,
    ["b"]                = bottom,
    ["t"]                = top
  }
end


local function PostUpdateHealthPower(self, event, unit, bar, min, max)
  bar.max = max;
  if oUF_Drathal_HUD.smooth == nil then
    DHUD_setValue(bar,min,max)
  end
  DHUD_setColor(self,bar,min,max)
end

function DHUD_setColor(self,bar,min,max)
  if unit == "player" then
    coff = 1
  else
    coff = 0.7
  end  
  
  if bar.type == "health" then
    local colors = self.colors.smoothHealth 
    local r, g, b = self.ColorGradient(min / max, unpack(colors))
    bar.Arc:SetVertexColor(r*coff, g*coff, b*coff)
  else
    local r, g, b = bar:GetStatusBarColor()
    bar.Arc:SetVertexColor(r*coff, g*coff, b*coff)
  end
end

function DHUD_setValue(bar,min,max)

  local t1,t2
  if bar.type == "health" then
    t1,t2 = 0,1   
  else
    t1,t2 = 1,0      
  end
  
  if (min == 0) then min = 1 end
  if (max == 0) then max = 1 end
  
  local c = CalcArc(bar.unit,min,max);
  bar.Arc:ClearAllPoints()
  bar.Arc:SetPoint("BOTTOM",0,c.gapBottom)
  bar.Arc:SetTexCoord(t1,t2, c.t, c.b )
  bar.Arc:SetHeight(c.height)
  
end

local function castColor(self, event, unit)
	local castbar = self.Castbar
	if(castbar.interrupt) then
		self.Castbar.Text:SetTextColor(0,0.8,0);
	else
    self.Castbar.Text:SetTextColor(1,1,1);
	end
end

local function updateCast(self, elapsed)

	if self.casting or self.channeling then  
    local duration = self.duration + elapsed
  
    self:SetMinMaxValues(0, self.max)        
		self:SetValue(duration)
    
    local castunit = (self.unit).."cast"
    local c = CalcArc( castunit,duration,self.max);
    self.Arc:ClearAllPoints()
    self.Arc:SetPoint("BOTTOM",0,c.gapBottom)
    self.Arc:SetTexCoord(1,0, c.t, c.b )
    self.Arc:SetHeight(c.height)    
  end

	if self.casting then
		local duration = self.duration + elapsed
		if (duration >= self.max) then
			self.casting = nil
			return
		end
    
		if self.Time then
			if self.delay ~= 0 then
				self.Time:SetFormattedText("%.1f|cffff0000-%.1f|r", duration, self.delay)
			else
				self.Time:SetFormattedText("%.1f", duration)
			end
		end
    
    self.duration = duration
    
	elseif self.channeling then
		local duration = self.duration - elapsed
		if(duration <= 0) then
			self.channeling = nil
			return
		end
    
		if self.Time then
			if self.delay ~= 0 then
				self.Time:SetFormattedText("%.1f|cffff0000-%.1f|r", duration, self.delay)
			else
				self.Time:SetFormattedText("%.1f", duration)
			end
		end

		self.duration = duration
    
  end
end

local function createArc(self,bar,type)
  
  local gap  = oUF_Drathal_HUD.spacing
  local tex1 = oUF_Drathal_HUD_Layout.fg
  local tex2 = oUF_Drathal_HUD_Layout.bg
  
  if type == "health" then
    gap = -oUF_Drathal_HUD.spacing;
  end
  
  if self.unit == "target" then    
    tex1 = oUF_Drathal_HUD_Layout.fg2
    tex2 = oUF_Drathal_HUD_Layout.bg2   
  end
  
  --[[ Statusbar invisible ]]--
  bar:SetStatusBarTexture(oUF_Drathal_HUD_Layout.blank) 
  bar:SetPoint("BOTTOM",gap,0)
  bar:SetHeight(oUF_Drathal_HUD_Layout.arcHeight)
  bar:SetWidth(oUF_Drathal_HUD_Layout.arcWidth)
  bar:SetOrientation("VERTICAL")
  bar.frequentUpdates = true
	bar.colorSmooth     = true
  bar.type = type
  bar.unit = self.unit
  
  if type == "castbar" then
    tex1 = oUF_Drathal_HUD_Layout.ecb
    if self.unit == "target" then
      tex1 = oUF_Drathal_HUD_Layout.ecb2
    end
        
    self.OnCastbarUpdate = updateCast
  end  

  --[[ Bar ]]--
	local arc = bar:CreateTexture(nil, 'ARTWORK')
  arc:SetPoint("BOTTOM",0,0)  
  arc:SetWidth(oUF_Drathal_HUD_Layout.arcWidth)
	arc:SetTexture(tex1)
  if type == "power" or type == "castbar" then
    arc:SetTexCoord(1,0,0, 1 )
  end  
  bar.Arc = arc

  --[[ Bar Background ]]--
  if type ~= "castbar" then    
    local arcbg = bar:CreateTexture(nil, 'BACKGROUND')
    arcbg:SetAllPoints(bar)
    arcbg:SetTexture(tex2)
    arcbg:SetVertexColor(0.8, 0.8, 0.8,1)
    if type == "power" then
      arcbg:SetTexCoord(1,0,0, 1 )
    end
    bar.back = arcbg
  end
  
end

local RAID_TARGET_UPDATE = function(self, event)
 self.Health.Text:UpdateTag()
end

local OverrideUpdateThreat = function(self, event, unit)
  if not self then return end
	unit = unit or self.unit

	local status = UnitThreatSituation(unit)
	if(status and  (status == 2 or status == 3) ) then
		self.Health.back:SetVertexColor(1,0,0)
    self.Power.back:SetVertexColor(1,0,0)
  else
    self.Health.back:SetVertexColor(0.8,0.8,0.8)
    self.Power.back:SetVertexColor(0.8,0.8,0.8)
	end	
end

local function styleFunc(self, unit)

  self:EnableMouse(false)
	self:SetAttribute('initial-height', oUF_Drathal_HUD_Layout.arcHeight)
	self:SetAttribute('initial-width',  oUF_Drathal_HUD_Layout.arcWidth)
  self.PostUpdateHealth    = PostUpdateHealthPower
  self.PostUpdatePower     = PostUpdateHealthPower
  self.disallowVehicleSwap = false;
  
  local level;
  if unit == "target" then
    level = 0;
  else
    level = 200;
  end

  -- Health Bar
  self.Health = CreateFrame('StatusBar', nil, self)
  self.Health:SetFrameLevel(level+10)
  createArc(self,self.Health,"health")
  if oUF_Drathal_HUD.smooth then
    self.Health.Smooth = true
  end
  
  -- Power Bar
  self.Power = CreateFrame('StatusBar', nil, self)  
  self.Power.colorPower = true
  self.Power:SetFrameLevel(level+20)
  createArc(self,self.Power,"power")
  if oUF_Drathal_HUD.smooth then
    self.Power.Smooth = true
  end
      
  -- castbar & castbar text
  if oUF_Drathal_HUD.show_targetcastbar and unit == "target" then
    self.Castbar = CreateFrame('StatusBar', nil, self)
    self.Castbar:SetFrameStrata("HIGH")
    self.Castbar:SetFrameLevel(level+30)
    createArc(self,self.Castbar,"castbar")
    self.Castbar.Arc:SetVertexColor(unpack(oUF_Drathal_HUD.castbarcolor));
    if oUF_Drathal_HUD.show_targetcastbartext then
      self.Castbar.Text = setFontString(self.Castbar, oUF_Drathal_HUD_Layout.font, 15,"THINOUTLINE")
      self.Castbar.Text:SetPoint('CENTER', self.Castbar, -210, -158)
    end
  end
  
  if oUF_Drathal_HUD.show_playercastbar and unit == "player" then
    self.Castbar = CreateFrame('StatusBar', nil, self)
    self.Castbar:SetFrameStrata("HIGH")
    self.Castbar:SetFrameLevel(level+35)
    createArc(self,self.Castbar,"castbar")
    self.Castbar.Arc:SetVertexColor(unpack(oUF_Drathal_HUD.castbarcolor));
    if oUF_Drathal_HUD.show_playercastbartext then
      self.Castbar.Text = setFontString(self.Castbar, oUF_Drathal_HUD_Layout.font, 15,"THINOUTLINE")
      self.Castbar.Text:SetPoint('CENTER', self.Castbar, -210, 158)
    end
  end  
    
  if self.Castbar and self.Castbar.Text then
    -- target Castbar Time
    self.Castbar.Time = setFontString(self.Castbar, oUF_Drathal_HUD_Layout.font, 15,"THINOUTLINE")		
    self.Castbar.Time:SetPoint('LEFT', self.Castbar.Text, 'RIGHT', 0, 0)
    
    -- target Castbar Button
    self.Castbar.Button = CreateFrame('Frame', nil, self.Castbar)
    self.Castbar.Button:SetHeight(18)
    self.Castbar.Button:SetWidth(18)
    self.Castbar.Button:SetBackdrop(backdrop)
    self.Castbar.Button:SetBackdropColor(0, 0, 0)
    self.Castbar.Button:SetPoint('RIGHT', self.Castbar.Text, 'LEFT', 0, 0)     
    self.Castbar.Icon = self.Castbar.Button:CreateTexture(nil, 'ARTWORK')
    self.Castbar.Icon:SetAllPoints(self.Castbar.Button)
    self.Castbar.Icon:SetTexCoord(0.08, 0.92, 0.08, 0.92)
  end
  

  if unit == "player" then
    -- threat coloring
    self.OverrideUpdateThreat = OverrideUpdateThreat
    self.Threat = self.Health:CreateTexture(nil, 'OVERLAY');
    
    -- resting icon
    self.Resting = self.Health:CreateTexture(nil, 'OVERLAY')
    self.Resting:SetPoint('TOP', self.Health, 40, 8)
    self.Resting:SetWidth(25)
    self.Resting:SetHeight(25)
    
    -- combat feedback
		self.CombatFeedbackText = self.Health:CreateFontString(nil, 'ARTWORK')
		self.CombatFeedbackText:SetFont(oUF_Drathal_HUD_Layout.font, 20, 'OUTLINE')
		self.CombatFeedbackText:SetShadowOffset(0, 0)
		self.CombatFeedbackText:SetPoint('CENTER', self.Health,'CENTER',70,0)	    
  end
  
  if unit == "target" then
		self.CombatFeedbackText = self.Health:CreateFontString(nil, 'ARTWORK')
		self.CombatFeedbackText:SetFont(oUF_Drathal_HUD_Layout.font, 20, 'OUTLINE')
		self.CombatFeedbackText:SetShadowOffset(0, 0)
		self.CombatFeedbackText:SetPoint('CENTER', self.Health,'CENTER',-60,0)	   
  
  
    -- target Alpha
    self.inRangeAlphaHUD = 1
    self.outsideRangeAlphaHUD = 0.7
    self.SpellRangeHUD = true
    
    -- target Text
    if oUF_Drathal_HUD.show_targettext then
      self.Health.Text = setFontString(self.Health, oUF_Drathal_HUD_Layout.font, 15,"THINOUTLINE")
      self.Health.Text:SetPoint('CENTER', self.Health, oUF_Drathal_HUD.spacing, -138)
      self:Tag(self.Health.Text,oUF_Drathal_HUD.targettext)    
      self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
      table.insert(self.__elements, RAID_TARGET_UPDATE)
    end
        
    -- Colorize casttext when interruptable
    if self.Castbar and self.Castbar.Text then
      self.PostCastStart    = castColor
      self.PostChannelStart = castColor
    end
    
    -- Rare / Elite
    self.RareElite = self.Health:CreateTexture(nil,"ARTWORK");
    self.RareElite:SetWidth(64);
    self.RareElite:SetHeight(64);
    self.RareElite:SetPoint("TOPLEFT",self.Health,"TOPLEFT",50,22);    
    
    -- COMBO Points
    if (select(2, UnitClass("player")) == "ROGUE" or select(2, UnitClass("player")) == "DRUID") then
      local bg = {}
      self.CPoints = {}
      for i = 1, MAX_COMBO_POINTS do
        self.CPoints[i] = CreateFrame("Frame", nil, self.Health)        
        self.CPoints[i]:SetSize(16, 16)
        if i > 1 then
          self.CPoints[i]:SetPoint("BOTTOM", self.CPoints[i-1], "TOP",oUF_Drathal_HUD_Layout.cpX[i],0)
        end
        bg[i] = self.CPoints[i]:CreateTexture(nil, "LOW")
        bg[i]:SetTexture(oUF_Drathal_HUD_Layout.cp[i])
        bg[i]:SetAllPoints(self.CPoints[i])
      end
      self.CPoints[1]:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT",-135,0)
       
    end
  end

end

oUF:RegisterStyle('hud', styleFunc)
oUF:SetActiveStyle('hud')

if oUF_Drathal_HUD.show_playerArc then
  oUF:Spawn('player','oUF_Player_HUD'):SetPoint('CENTER', UIParent, 'CENTER', oUF_Drathal_HUD.xoffset, oUF_Drathal_HUD.yoffset)
  oUF_Player_HUD:SetScale(oUF_Drathal_HUD.scale)
end

if oUF_Drathal_HUD.show_targetArc then
  oUF:Spawn('target','oUF_Target_HUD'):SetPoint('CENTER', UIParent, 'CENTER', oUF_Drathal_HUD.xoffset, oUF_Drathal_HUD.yoffset)
  oUF_Target_HUD:SetScale(oUF_Drathal_HUD.scale)
end
